@@ -37,6 +37,7 @@
 #include <ctype.h>
 #include <regex.h>
 #include <limits.h>
+#include <math.h>
 
 
 #include "NumberFormatException.h"
@@ -53,14 +54,6 @@
  */
 
 
-/* ----------------------------------------------------------- Definitions */
-
-
-static const char *kSizeNotation[9] = {
-        "B", "kB", "MB", "GB", "TB", "PB", "EB", "ZB", NULL
-};
-
-
 /* -------------------------------------------------------- Public Methods */
 
 
@@ -86,7 +79,7 @@ char *Str_ltrim(char *s) {
                 for (j = 0; s[j]; j++) ;
                 for (i = 0; isspace(s[i]); i++) ;
                 memmove(s, s + i, j - i);
-                s[j-i] = 0;
+                s[j - i] = 0;
         }
         return s;
 }
@@ -199,8 +192,8 @@ int Str_startsWith(const char *a, const char *b) {
 int Str_endsWith(const char *a, const char *b) {
         if (a && b) {
                 size_t i = 0, j = 0;
-                for(i = strlen(a), j = strlen(b); (i && j); i--, j--)
-                        if(toupper(a[i]) != toupper(b[j])) return false;
+                for (i = strlen(a), j = strlen(b); (i && j); i--, j--)
+                        if (toupper(a[i]) != toupper(b[j])) return false;
                 return (i >= j);
         }
         return false;
@@ -239,8 +232,27 @@ int Str_has(const char *charset, const char *s) {
 }
 
 
+char *Str_unescape(const char *charset, char *s) {
+        if (charset && STR_DEF(s)) {
+                int x, y;
+                for (x = 0, y = 0; s[y]; x++, y++) {
+                        if ((s[x] = s[y]) == '\\')
+                                for (int i = 0; charset[i]; i++) {
+                                        if (charset[i] == s[y + 1]) {
+                                                s[x] = charset[i];
+                                                y++;
+                                                break;
+                                        }
+                                }
+                }
+                s[x] = 0;
+        }
+        return s;
+}
+
+
 int Str_isEqual(const char *a, const char *b) {
-        if (a && b) { 
+        if (a && b) {
                 while (*a && *b)
                         if (toupper(*a++) != toupper(*b++)) return false;
                 return (*a == *b);
@@ -409,18 +421,50 @@ int Str_cmp(const void *x, const void *y) {
 }
 
 
+int Str_compareConstantTime(const void *x, const void *y) {
+        // Copy input to zero initialized buffers of fixed size, to prevent string length timing attack (handle NULL input as well). If some string exceeds hardcoded buffer size, error is returned.
+        char _x[MAX_CONSTANT_TIME_STRING_LENGTH + 1] = {};
+        char _y[MAX_CONSTANT_TIME_STRING_LENGTH + 1] = {};
+        if (snprintf(_x, sizeof(_x), "%s", x ? (const char *)x : "") > MAX_CONSTANT_TIME_STRING_LENGTH || snprintf(_y, sizeof(_y), "%s", y ? (const char *)y : "") > MAX_CONSTANT_TIME_STRING_LENGTH) 
+                return 1;
+        int rv = 0;
+        for (size_t i = 0; i < sizeof(_x); i++)
+                rv |= _x[i] ^ _y[i];
+        return rv;
+}
+
+
 char *Str_bytesToSize(double bytes, char s[10]) {
         assert(s);
         assert(bytes < 1e+24);
+        static const char *kNotation[] = {"B", "kB", "MB", "GB", "TB", "PB", "EB", "ZB", NULL};
         *s = 0;
-        for (int i = 0; kSizeNotation[i]; i++) {
+        for (int i = 0; kNotation[i]; i++) {
                 if (bytes > 1024) {
                         bytes /= 1024;
                 } else {
-                        snprintf(s, 10, "%.1lf %s", bytes, kSizeNotation[i]);
+                        snprintf(s, 10, (round(bytes) == bytes) ? "%.0lf %s" : "%.1lf %s", bytes, kNotation[i]);
                         break;
                 }
         }
         return s;
 }
 
+
+char *Str_milliToTime(double milli, char s[23]) {
+        assert(s);
+        struct conversion {
+                double base;
+                char *suffix;
+        } conversion[]= {
+                {1000., "ms"}, // millisecond
+                {60.,   "s"},  // second
+                {60.,   "m"}   // minute
+        };
+        int index = 0;
+        while (fabs(milli) >= conversion[index].base && index < sizeof(conversion) / sizeof(conversion[0]) - 1)
+                milli /= conversion[index++].base;
+        snprintf(s, 23, (round(milli) == milli) ? "%.0lf %s" : "%.3lf %s", milli, conversion[index].suffix);
+        return s;
+}
+
