@@ -37,6 +37,13 @@
  */
 
 
+/**
+ * Maximum length of input for Str_compareConstantTime() method. We support
+ * currently up to 64 characters, which is enough for SHA256 digests.
+ */
+#define MAX_CONSTANT_TIME_STRING_LENGTH 64
+
+
 /**
  * Test if the given string is defined. That is; not NULL nor the 
  * empty ("") string
@@ -214,6 +221,23 @@ char *Str_sub(const char *a, const char *b);
 int Str_has(const char *charset, const char *s);
 
 
+/**
+ * Unescape all characters in <code>s</code> which are in the 
+ * <code>charset</code> and return <code>s</code> modified.
+ * Example:
+ * <pre>
+ * char s[] = "foo\'ba\"r\}baz";
+ * Str_unescape("\"'", s) -> foo'ba"r\}baz
+ * </pre>
+ * @param charset The characters to test <code>s</code> against. 
+ * A character is unescaped in <code>s</code> if it is in the 
+ * charset and is preceded with '\'.
+ * @param s The string to unescape
+ * @return A pointer to s
+ */
+char *Str_unescape(const char *charset, char *s);
+
+
 /**
  * Returns true if <i>a</i> equals <i>b</i>. The test is 
  * <i>case-insensitive</i> but depends on that all characters
@@ -408,6 +432,16 @@ unsigned int Str_hash(const void *x);
 int Str_cmp(const void *x, const void *y);
 
 
+/**
+ * Compare case sensitive two strings in constant time. This function
+ * can be used for timing-attack resistent comparison of credentials.
+ * @param x A String
+ * @param y A String
+ * @return 0 if x and y are equal otherwise a non-zero integer
+ */
+int Str_compareConstantTime(const void *x, const void *y);
+
+
 /**
  * Convert the numeric bytes value to a string representation scaled to
  * human friendly storage unit [B, kB, MB, etc.].
@@ -417,5 +451,15 @@ int Str_cmp(const void *x, const void *y);
  */
 char *Str_bytesToSize(double bytes, char s[10]);
 
+
+/**
+ * Convert the value in milliseconds to human friendlier unit (ms/s/m).
+ * @param milli The time value in milliseconds to present
+ * @param s A result buffer, must be large enough to hold 23 chars
+ * @return A pointer to s
+ */
+char *Str_milliToTime(double milli, char s[23]);
+
+
 #endif
 
