@@ -263,8 +263,8 @@ int main(void) {
         }
         printf("=> Test11: OK\n\n");
         
-#if ! defined(OPENBSD)
-        /* FIXME: MONIT-35: OpenBSD vfork() doesn't share memory, so the current implementation of Command_execute() cannot pass the execve() error to parent. Disable the unit test on OpenBSD for now. */
+#if ! defined(OPENBSD) && ! defined(AIX)
+        /* FIXME: MONIT-35: OpenBSD and AIX vfork() doesn't share memory, so the current implementation of Command_execute() cannot pass the execve() error to parent. Disable the unit test on OpenBSD and AIX for now. */
         printf("=> Test12: on execute error\n");
         {
                 // Try executing a directory should produce an error
