@@ -25,7 +25,7 @@
 #ifndef MONIT_DEVICE_H
 #define MONIT_DEVICE_H
 
-int filesystem_usage(Service_T);
+boolean_t filesystem_usage(Service_T);
 
 #endif
 
