@@ -109,8 +109,8 @@
  *  @file
  */
 
-/* There is no prototype for getprocs64 in AIX <= 5.3 */
 int getprocs64(void *, int, void *, int, pid_t *, int);
+int getargs(struct procentry64 *processBuffer, int bufferLen, char *argsBuffer, int argsLen);
 
 static int                page_size;
 static int                cpu_initialized = 0;
@@ -119,8 +119,6 @@ static unsigned long long cpu_user_old  = 0ULL;
 static unsigned long long cpu_syst_old  = 0ULL;
 static unsigned long long cpu_wait_old  = 0ULL;
 
-struct procentry64 *procs = NULL;
-
 
 boolean_t init_process_info_sysdep(void) {
         perfstat_memory_total_t mem;
@@ -130,9 +128,9 @@ boolean_t init_process_info_sysdep(void) {
                 return false;
         }
 
-        page_size                = getpagesize();
-        systeminfo.mem_kbyte_max = (unsigned long)(mem.real_total * (page_size / 1024));
-        systeminfo.cpus          = sysconf(_SC_NPROCESSORS_ONLN);
+        page_size          = getpagesize();
+        systeminfo.mem_max = mem.real_total * page_size;
+        systeminfo.cpus    = sysconf(_SC_NPROCESSORS_ONLN);
 
         return true;
 }
@@ -171,22 +169,18 @@ int getloadavg_sysdep (double *loadv, int nelem) {
 /**
  * Read all processes to initialize the process tree
  * @param reference  reference of ProcessTree
- * @return treesize>0 if succeeded otherwise =0.
+ * @param pflags Process engine flags
+ * @return treesize > 0 if succeeded otherwise 0.
  */
-int initprocesstree_sysdep(ProcessTree_T ** reference) {
-        int             treesize;
-        struct userinfo user;
-        ProcessTree_T  *pt;
-        pid_t           firstproc = 0;
-
-        memset(&user, 0, sizeof(struct userinfo));
-
+int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags) {
+        int treesize;
+        pid_t firstproc = 0;
         if ((treesize = getprocs64(NULL, 0, NULL, 0, &firstproc, PID_MAX)) < 0) {
                 LogError("system statistic error -- getprocs64 failed: %s\n", STRERROR);
                 return 0;
         }
 
-        procs = CALLOC(sizeof(struct procentry64), treesize);
+        struct procentry64 *procs = CALLOC(sizeof(struct procentry64), treesize);
 
         firstproc = 0;
         if ((treesize = getprocs64(procs, sizeof(struct procentry64), NULL, 0, &firstproc, treesize)) < 0) {
@@ -195,44 +189,56 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
                 return 0;
         }
 
-        pt = CALLOC(sizeof(ProcessTree_T), treesize);
+        ProcessTree_T *pt = CALLOC(sizeof(ProcessTree_T), treesize);
 
         for (int i = 0; i < treesize; i++) {
-                int fd;
-                struct psinfo ps;
+                pt[i].pid          = procs[i].pi_pid;
+                pt[i].ppid         = procs[i].pi_ppid;
+                pt[i].cred.euid    = procs[i].pi_uid;
+                pt[i].threads      = procs[i].pi_thcount;
+                pt[i].uptime       = systeminfo.time / 10. - procs[i].pi_start;
+                pt[i].memory.usage = (procs[i].pi_drss + procs[i].pi_trss) * page_size;
+                pt[i].cpu.time     = procs[i].pi_ru.ru_utime.tv_sec * 10 + (double)procs[i].pi_ru.ru_utime.tv_usec / 100000. + procs[i].pi_ru.ru_stime.tv_sec * 10 + (double)procs[i].pi_ru.ru_stime.tv_usec / 100000.;
+                pt[i].zombie       = procs[i].pi_state == SZOMB ? true: false;
+
                 char filename[STRLEN];
-
-                pt[i].cputime     = 0;
-                pt[i].cpu_percent = 0;
-                pt[i].mem_kbyte   = 0;
-                pt[i].pid         = procs[i].pi_pid;
-                pt[i].ppid        = procs[i].pi_ppid;
-                pt[i].starttime   = procs[i].pi_start;
-
-                if (procs[i].pi_state == SZOMB) {
-                        pt[i].zombie = true;
-                } else if (getuser(&(procs[i]), sizeof(struct procinfo), &user, sizeof(struct userinfo)) != -1) {
-                        pt[i].mem_kbyte = (user.ui_drss + user.ui_trss) * (page_size / 1024);
-                        pt[i].cputime   = (user.ui_ru.ru_utime.tv_sec + user.ui_ru.ru_utime.tv_usec * 1.0e-6 + user.ui_ru.ru_stime.tv_sec + user.ui_ru.ru_stime.tv_usec * 1.0e-6) * 10;
-                }
-
                 snprintf(filename, sizeof(filename), "/proc/%d/psinfo", pt[i].pid);
-                if ((fd = open(filename, O_RDONLY)) < 0) {
+                int fd = open(filename, O_RDONLY);
+                if (fd < 0) {
                         DEBUG("Cannot open proc file %s -- %s\n", filename, STRERROR);
                         continue;
                 }
+                struct psinfo ps;
                 if (read(fd, &ps, sizeof(ps)) < 0) {
                         DEBUG("Cannot read proc file %s -- %s\n", filename, STRERROR);
                         if (close(fd) < 0)
                                 LogError("Socket close failed -- %s\n", STRERROR);
-                        return 0;
+                        continue;
                 }
                 if (close(fd) < 0)
                         LogError("Socket close failed -- %s\n", STRERROR);
-                pt[i].uid     = ps.pr_uid;
-                pt[i].euid    = ps.pr_euid;
-                pt[i].gid     = ps.pr_gid;
-                pt[i].cmdline = (ps.pr_psargs && *ps.pr_psargs) ? Str_dup(ps.pr_psargs) : Str_dup(procs[i].pi_comm);
+                pt[i].cred.uid = ps.pr_uid;
+                pt[i].cred.gid = ps.pr_gid;
+                if (pflags & ProcessEngine_CollectCommandLine) {
+                        if (ps.pr_argc == 0) {
+                                pt[i].cmdline = Str_dup(procs[i].pi_comm); // Kernel thread
+                        } else {
+                                char command[4096];
+                                if (! getargs(&procs[i], sizeof(struct procentry64), command, sizeof(command))) {
+                                        // The arguments are separated with '\0' with the last one terminated by '\0\0' -> merge arguments into one string
+                                        for (int i = 0; i < sizeof(command) - 1; i++) {
+                                                if (command[i] == '\0') {
+                                                        if (command[i + 1] == '\0')
+                                                                break;
+                                                        command[i] = ' ';
+                                                }
+                                        }
+                                        pt[i].cmdline = Str_dup(command);
+                                } else {
+                                        pt[i].cmdline = Str_dup(procs[i].pi_comm);
+                                }
+                        }
+                }
         }
 
         FREE(procs);
@@ -254,11 +260,11 @@ boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
                 LogError("system statistic error -- perfstat_memory_total failed: %s\n", STRERROR);
                 return false;
         }
-        si->total_mem_kbyte = (unsigned long)((mem.real_total - mem.real_free - mem.numperm) * (page_size / 1024));
+        si->total_mem = (mem.real_total - mem.real_free - mem.numperm) * page_size;
 
         /* Swap */
-        si->swap_kbyte_max   = (unsigned long)(mem.pgsp_total * 4);                   /* 4kB blocks */
-        si->total_swap_kbyte = (unsigned long)((mem.pgsp_total - mem.pgsp_free) * 4); /* 4kB blocks */
+        si->swap_max   = mem.pgsp_total * 4096;                   /* 4kB blocks */
+        si->total_swap = (mem.pgsp_total - mem.pgsp_free) * 4096; /* 4kB blocks */
 
         return true;
 }
@@ -290,13 +296,13 @@ boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
 
         if (cpu_initialized) {
                 if (cpu_total > 0) {
-                        si->total_cpu_user_percent = 1000 * ((double)(cpu_user - cpu_user_old) / (double)cpu_total);
-                        si->total_cpu_syst_percent = 1000 * ((double)(cpu_syst - cpu_syst_old) / (double)cpu_total);
-                        si->total_cpu_wait_percent = 1000 * ((double)(cpu_wait - cpu_wait_old) / (double)cpu_total);
+                        si->total_cpu_user_percent = 100. * ((double)(cpu_user - cpu_user_old) / (double)cpu_total);
+                        si->total_cpu_syst_percent = 100. * ((double)(cpu_syst - cpu_syst_old) / (double)cpu_total);
+                        si->total_cpu_wait_percent = 100. * ((double)(cpu_wait - cpu_wait_old) / (double)cpu_total);
                 } else {
-                        si->total_cpu_user_percent = 0;
-                        si->total_cpu_syst_percent = 0;
-                        si->total_cpu_wait_percent = 0;
+                        si->total_cpu_user_percent = 0.;
+                        si->total_cpu_syst_percent = 0.;
+                        si->total_cpu_wait_percent = 0.;
                 }
         }
 
