@@ -69,8 +69,8 @@
 #include <glob.h>
 #endif
 
-#ifndef HZ
-# define HZ sysconf(_SC_CLK_TCK)
+#ifdef HAVE_SYS_SYSINFO_H
+#include <sys/sysinfo.h>
 #endif
 
 #include "monit.h"
@@ -90,44 +90,28 @@
 /* ----------------------------------------------------------------- Private */
 
 
-#define UID             "Uid:"
-#define GID             "Gid:"
-#define MEMTOTAL        "MemTotal:"
-#define MEMFREE         "MemFree:"
-#define MEMBUF          "Buffers:"
-#define MEMCACHE        "Cached:"
-#define SLABRECLAIMABLE "SReclaimable:"
-#define SWAPTOTAL       "SwapTotal:"
-#define SWAPFREE        "SwapFree:"
-
 #define NSEC_PER_SEC    1000000000L
 
 static unsigned long long old_cpu_user     = 0;
 static unsigned long long old_cpu_syst     = 0;
 static unsigned long long old_cpu_wait     = 0;
 static unsigned long long old_cpu_total    = 0;
-static int                page_shift_to_kb = 0;
 
+static long page_size = 0;
+
+static double hz = 0.;
 
 /**
  * Get system start time
  * @return seconds since unix epoch
  */
 static time_t get_starttime() {
-        char   buf[STRLEN];
-        double up = 0;
-
-        if (! read_proc_file(buf, sizeof(buf), "uptime", -1, NULL)) {
-                LogError("system statistic error -- cannot get system uptime\n");
+        struct sysinfo info;
+        if (sysinfo(&info) < 0) {
+                LogError("system statistic error -- cannot get system uptime: %s\n", STRERROR);
                 return 0;
         }
-
-        if (sscanf(buf, "%lf", &up) != 1) {
-                LogError("system statistic error -- invalid uptime\n");
-                return 0;
-        }
-
-        return Time_now() - (time_t)up;
+        return Time_now() - info.uptime;
 }
 
 
@@ -137,21 +121,31 @@ static time_t get_starttime() {
 boolean_t init_process_info_sysdep(void) {
         char *ptr;
         char  buf[2048];
-        long  page_size;
-        int   page_shift;
+
+        if ((hz = sysconf(_SC_CLK_TCK)) <= 0.) {
+                DEBUG("system statistic error -- cannot get hz: %s\n", STRERROR);
+                return false;
+        }
+
+        if ((page_size = sysconf(_SC_PAGESIZE)) <= 0) {
+                DEBUG("system statistic error -- cannot get page size: %s\n", STRERROR);
+                return false;
+        }
 
         if (! read_proc_file(buf, sizeof(buf), "meminfo", -1, NULL)) {
                 DEBUG("system statistic error -- cannot read /proc/meminfo\n");
                 return false;
         }
-        if (! (ptr = strstr(buf, MEMTOTAL))) {
+        if (! (ptr = strstr(buf, "MemTotal:"))) {
                 DEBUG("system statistic error -- cannot get real memory amount\n");
                 return false;
         }
-        if (sscanf(ptr+strlen(MEMTOTAL), "%ld", &systeminfo.mem_kbyte_max) != 1) {
+        long mem_max;
+        if (sscanf(ptr + 9, "%ld", &mem_max) != 1) {
                 DEBUG("system statistic error -- cannot get real memory amount\n");
                 return false;
         }
+        systeminfo.mem_max = mem_max * 1024;
 
         if ((systeminfo.cpus = sysconf(_SC_NPROCESSORS_CONF)) < 0) {
                 DEBUG("system statistic error -- cannot get cpu count: %s\n", STRERROR);
@@ -161,27 +155,17 @@ boolean_t init_process_info_sysdep(void) {
                 systeminfo.cpus = 1;
         }
 
-        if ((page_size = sysconf(_SC_PAGESIZE)) <= 0) {
-                DEBUG("system statistic error -- cannot get page size: %s\n", STRERROR);
-                return false;
-        }
-
-        for (page_shift = 0; page_size != 1; page_size >>= 1, page_shift++)
-                ;
-        page_shift_to_kb = page_shift - 10;
-
         return true;
 }
 
 
 /**
- * Read all processes of the proc files system to initialize
- * the process tree (sysdep version... but should work for
- * all procfs based unices)
- * @param reference  reference of ProcessTree
- * @return treesize>0 if succeeded otherwise =0.
+ * Read all processes of the proc files system to initialize the process tree
+ * @param reference reference of ProcessTree
+ * @param pflags Process engine flags
+ * @return treesize > 0 if succeeded otherwise 0
  */
-int initprocesstree_sysdep(ProcessTree_T ** reference) {
+int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags) {
         int                 rv, bytes = 0;
         int                 treesize = 0;
         int                 stat_pid = 0;
@@ -196,27 +180,28 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
         long                stat_item_cutime = 0;
         long                stat_item_cstime = 0;
         long                stat_item_rss = 0;
+        int                 stat_item_threads = 0;
         glob_t              globbuf;
         unsigned long       stat_item_utime = 0;
         unsigned long       stat_item_stime = 0;
         unsigned long long  stat_item_starttime = 0ULL;
-        ProcessTree_T      *pt = NULL;
 
         ASSERT(reference);
 
         /* Find all processes in the /proc directory */
-        if ((rv = glob("/proc/[0-9]*", GLOB_ONLYDIR, NULL, &globbuf))) {
+        if ((rv = glob("/proc/[0-9]*", 0, NULL, &globbuf))) {
                 LogError("system statistic error -- glob failed: %d (%s)\n", rv, STRERROR);
                 return 0;
         }
 
         treesize = globbuf.gl_pathc;
 
-        pt = CALLOC(sizeof(ProcessTree_T), treesize);
+        ProcessTree_T *pt = CALLOC(sizeof(ProcessTree_T), treesize);
 
         /* Insert data from /proc directory */
+        time_t starttime = get_starttime();
         for (int i = 0; i < treesize; i++) {
-                stat_pid = atoi(globbuf.gl_pathv[i] + strlen("/proc/"));
+                stat_pid = atoi(globbuf.gl_pathv[i] + 6); // skip "/proc/"
 
                 /********** /proc/PID/stat **********/
                 if (! read_proc_file(buf, sizeof(buf), "stat", stat_pid, NULL)) {
@@ -228,24 +213,22 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
                         continue;
                 }
                 *tmp = 0;
-                if (sscanf(buf, "%*d (%256s", procname) != 1) {
+                if (sscanf(buf, "%*d (%255s", procname) != 1) {
                         DEBUG("system statistic error -- file /proc/%d/stat process name parse error\n", stat_pid);
                         continue;
                 }
                 tmp += 2;
                 if (sscanf(tmp,
-                           "%c %d %*d %*d %*d %*d %*u %*u"
-                           "%*u %*u %*u %lu %lu %ld %ld %*d %*d %*d "
-                           "%*u %llu %*u %ld %*u %*u %*u %*u %*u "
-                           "%*u %*u %*u %*u %*u %*u %*u %*u %*d %*d\n",
+                           "%c %d %*d %*d %*d %*d %*u %*u %*u %*u %*u %lu %lu %ld %ld %*d %*d %d %*u %llu %*u %ld %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*d %*d\n",
                            &stat_item_state,
                            &stat_ppid,
                            &stat_item_utime,
                            &stat_item_stime,
                            &stat_item_cutime,
                            &stat_item_cstime,
+                           &stat_item_threads,
                            &stat_item_starttime,
-                           &stat_item_rss) != 8) {
+                           &stat_item_rss) != 9) {
                         DEBUG("system statistic error -- file /proc/%d/stat parse error\n", stat_pid);
                         continue;
                 }
@@ -255,46 +238,46 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
                         DEBUG("system statistic error -- cannot read /proc/%d/status\n", stat_pid);
                         continue;
                 }
-                if (! (tmp = strstr(buf, UID))) {
+                if (! (tmp = strstr(buf, "Uid:"))) {
                         DEBUG("system statistic error -- cannot find process uid\n");
                         continue;
                 }
-                if (sscanf(tmp+strlen(UID), "\t%d\t%d", &stat_uid, &stat_euid) != 2) {
+                if (sscanf(tmp + 4, "\t%d\t%d", &stat_uid, &stat_euid) != 2) {
                         DEBUG("system statistic error -- cannot read process uid\n");
                         continue;
                 }
-                if (! (tmp = strstr(buf, GID))) {
+                if (! (tmp = strstr(buf, "Gid:"))) {
                         DEBUG("system statistic error -- cannot find process gid\n");
                         continue;
                 }
-                if (sscanf(tmp+strlen(GID), "\t%d", &stat_gid) != 1) {
+                if (sscanf(tmp + 4, "\t%d", &stat_gid) != 1) {
                         DEBUG("system statistic error -- cannot read process gid\n");
                         continue;
                 }
 
                 /********** /proc/PID/cmdline **********/
-                if (! read_proc_file(buf, sizeof(buf), "cmdline", stat_pid, &bytes)) {
-                        DEBUG("system statistic error -- cannot read /proc/%d/cmdline\n", stat_pid);
-                        continue;
+                if (pflags & ProcessEngine_CollectCommandLine) {
+                        if (! read_proc_file(buf, sizeof(buf), "cmdline", stat_pid, &bytes)) {
+                                DEBUG("system statistic error -- cannot read /proc/%d/cmdline\n", stat_pid);
+                                continue;
+                        }
+                        for (int j = 0; j < (bytes - 1); j++) // The cmdline file contains argv elements/strings terminated separated by '\0' => join the string
+                                if (buf[j] == 0)
+                                        buf[j] = ' ';
+                        pt[i].cmdline = Str_dup(*buf ? buf : procname);
                 }
-                for (int j = 0; j < (bytes - 1); j++) // The cmdline file contains argv elements/strings terminated separated by '\0' => join the string
-                        if (buf[j] == 0)
-                                buf[j] = ' ';
 
                 /* Set the data in ptree only if all process related reads succeeded (prevent partial data in the case that continue was called during data gathering) */
-                pt[i].time = get_float_time();
                 pt[i].pid = stat_pid;
                 pt[i].ppid = stat_ppid;
-                pt[i].uid = stat_uid;
-                pt[i].euid = stat_euid;
-                pt[i].gid = stat_gid;
-                pt[i].starttime = get_starttime() + (time_t)(stat_item_starttime / HZ);
-                pt[i].cmdline = Str_dup(*buf ? buf : procname);
-                pt[i].cputime = ((float)(stat_item_utime + stat_item_stime) * 10.0) / HZ; // jiffies -> seconds = 1 / HZ. HZ is defined in "asm/param.h" and it is usually 1/100s but on alpha system it is 1/1024s
-                pt[i].cpu_percent = 0;
-                pt[i].mem_kbyte = (page_shift_to_kb < 0) ? (stat_item_rss >> abs(page_shift_to_kb)) : (stat_item_rss << abs(page_shift_to_kb));
-                if (stat_item_state == 'Z') // State is Zombie -> then we are a Zombie ... clear or? (-:
-                        pt[i].zombie = true;
+                pt[i].cred.uid = stat_uid;
+                pt[i].cred.euid = stat_euid;
+                pt[i].cred.gid = stat_gid;
+                pt[i].threads = stat_item_threads;
+                pt[i].uptime = starttime > 0 ? (systeminfo.time / 10. - (starttime + (time_t)(stat_item_starttime / hz))) : 0;
+                pt[i].cpu.time = (double)(stat_item_utime + stat_item_stime) / hz * 10.; // jiffies -> seconds = 1/hz
+                pt[i].memory.usage = stat_item_rss * page_size;
+                pt[i].zombie = stat_item_state == 'Z' ? true : false;
         }
 
         *reference = pt;
@@ -331,7 +314,7 @@ int getloadavg_sysdep(double *loadv, int nelem) {
 
 
 /**
- * This routine returns kbyte of real memory in use.
+ * This routine returns real memory in use.
  * @return: true if successful, false if failed
  */
 boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
@@ -350,35 +333,35 @@ boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
         }
 
         /* Memory */
-        if (! (ptr = strstr(buf, MEMFREE)) || sscanf(ptr + strlen(MEMFREE), "%ld", &mem_free) != 1) {
+        if (! (ptr = strstr(buf, "MemFree:")) || sscanf(ptr + 8, "%ld", &mem_free) != 1) {
                 LogError("system statistic error -- cannot get real memory free amount\n");
                 goto error;
         }
-        if (! (ptr = strstr(buf, MEMBUF)) || sscanf(ptr + strlen(MEMBUF), "%ld", &buffers) != 1)
+        if (! (ptr = strstr(buf, "Buffers:")) || sscanf(ptr + 8, "%ld", &buffers) != 1)
                 DEBUG("system statistic error -- cannot get real memory buffers amount\n");
-        if (! (ptr = strstr(buf, MEMCACHE)) || sscanf(ptr + strlen(MEMCACHE), "%ld", &cached) != 1)
+        if (! (ptr = strstr(buf, "Cached:")) || sscanf(ptr + 7, "%ld", &cached) != 1)
                 DEBUG("system statistic error -- cannot get real memory cache amount\n");
-        if (! (ptr = strstr(buf, SLABRECLAIMABLE)) || sscanf(ptr + strlen(SLABRECLAIMABLE), "%ld", &slabreclaimable) != 1)
+        if (! (ptr = strstr(buf, "SReclaimable:")) || sscanf(ptr + 13, "%ld", &slabreclaimable) != 1)
                 DEBUG("system statistic error -- cannot get slab reclaimable memory amount\n");
-        si->total_mem_kbyte = systeminfo.mem_kbyte_max - mem_free - buffers - cached - slabreclaimable;
+        si->total_mem = systeminfo.mem_max - (mem_free + buffers + cached + slabreclaimable) * 1024;
 
         /* Swap */
-        if (! (ptr = strstr(buf, SWAPTOTAL)) || sscanf(ptr + strlen(SWAPTOTAL), "%ld", &swap_total) != 1) {
+        if (! (ptr = strstr(buf, "SwapTotal:")) || sscanf(ptr + 10, "%ld", &swap_total) != 1) {
                 LogError("system statistic error -- cannot get swap total amount\n");
                 goto error;
         }
-        if (! (ptr = strstr(buf, SWAPFREE)) || sscanf(ptr + strlen(SWAPFREE), "%ld", &swap_free) != 1) {
+        if (! (ptr = strstr(buf, "SwapFree:")) || sscanf(ptr + 9, "%ld", &swap_free) != 1) {
                 LogError("system statistic error -- cannot get swap free amount\n");
                 goto error;
         }
-        si->swap_kbyte_max   = swap_total;
-        si->total_swap_kbyte = swap_total - swap_free;
+        si->swap_max = swap_total * 1024;
+        si->total_swap = (swap_total - swap_free) * 1024;
 
         return true;
 
 error:
-        si->total_mem_kbyte = 0;
-        si->swap_kbyte_max = 0;
+        si->total_mem = 0ULL;
+        si->swap_max = 0ULL;
         return false;
 }
 
@@ -426,15 +409,15 @@ boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
         cpu_user  = cpu_user + cpu_nice;
 
         if (old_cpu_total == 0) {
-                si->total_cpu_user_percent = -10;
-                si->total_cpu_syst_percent = -10;
-                si->total_cpu_wait_percent = -10;
+                si->total_cpu_user_percent = -1.;
+                si->total_cpu_syst_percent = -1.;
+                si->total_cpu_wait_percent = -1.;
         } else {
                 unsigned long long delta = cpu_total - old_cpu_total;
 
-                si->total_cpu_user_percent = (int)(1000 * (double)(cpu_user - old_cpu_user) / delta);
-                si->total_cpu_syst_percent = (int)(1000 * (double)(cpu_syst - old_cpu_syst) / delta);
-                si->total_cpu_wait_percent = (int)(1000 * (double)(cpu_wait - old_cpu_wait) / delta);
+                si->total_cpu_user_percent = 100. * (double)(cpu_user - old_cpu_user) / delta;
+                si->total_cpu_syst_percent = 100. * (double)(cpu_syst - old_cpu_syst) / delta;
+                si->total_cpu_wait_percent = 100. * (double)(cpu_wait - old_cpu_wait) / delta;
         }
 
         old_cpu_user  = cpu_user;
@@ -444,9 +427,9 @@ boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
         return true;
 
 error:
-        si->total_cpu_user_percent = 0;
-        si->total_cpu_syst_percent = 0;
-        si->total_cpu_wait_percent = 0;
+        si->total_cpu_user_percent = 0.;
+        si->total_cpu_syst_percent = 0.;
+        si->total_cpu_wait_percent = 0.;
         return false;
 }
 
