@@ -39,7 +39,7 @@
 
 /* Escape zero i.e. '\0' in expect buffer with "\0" so zero can be tested in expect strings as "\0". If there are no '\0' in the buffer it is returned as it is */
 static char *_escapeZeroInExpectBuffer(char *s, int n) {
-        assert(n < EXPECT_BUFFER_MAX);
+        assert(n < Run.limits.sendExpectBuffer);
         int i, j;
         char t[n]; // VLA
         for (i = 0, j = 0; j < n; i++, j++) {
@@ -71,7 +71,7 @@ void check_generic(Socket_T socket) {
         if (Socket_getPort(socket))
                 g = ((Port_T)(Socket_getPort(socket)))->parameters.generic.sendexpect;
 
-        char *buf = CALLOC(sizeof(char), Run.expectbuffer + 1);
+        char *buf = CALLOC(sizeof(char), Run.limits.sendExpectBuffer + 1);
 
         while (g != NULL) {
 
@@ -93,14 +93,16 @@ void check_generic(Socket_T socket) {
                          timeout seconds on EOF we first read one byte to fill the socket's read
                          buffer and then set a low timeout on next read which reads remaining bytes
                          as well as wait on EOF */
-                        *buf = Socket_readByte(socket);
-                        if ((int8_t)*buf < 0) {
+                        int first_byte = Socket_readByte(socket);
+                        if (first_byte < 0) {
                                 FREE(buf);
                                 THROW(IOException, "GENERIC: error receiving data -- %s", STRERROR);
                         }
+                        *buf = first_byte;
+
                         int timeout = Socket_getTimeout(socket);
                         Socket_setTimeout(socket, 200);
-                        int n = Socket_read(socket, buf + 1, Run.expectbuffer - 1) + 1;
+                        int n = Socket_read(socket, buf + 1, Run.limits.sendExpectBuffer - 1) + 1;
                         buf[n] = 0;
                         if (n > 0)
                                 _escapeZeroInExpectBuffer(buf, n);
