@@ -30,6 +30,10 @@
 
 #include "protocol.h"
 
+// libmonit
+#include "exceptions/IOException.h"
+
+
 /**
  *  A simple Postfix SMTP access policy delegation protocol test
  *
@@ -46,39 +50,30 @@
  *
  *  @file
  */
-int check_postfix_policy(Socket_T socket) {
+void check_postfix_policy(Socket_T socket) {
 
-  char buf[STRLEN];
+        char buf[STRLEN];
 
-  ASSERT(socket);
+        ASSERT(socket);
 
-  if(socket_print(socket,
-    "request=smtpd_access_policy\n"
-    "protocol_state=RCPT\n"
-    "protocol_name=SMTP\n"
-    "sender=monit@foo.tld\n"
-    "recipient=monit@foo.tld\n"
-    "client_address=1.2.3.4\n"
-    "client_name=mx.foo.tld\n"
-    "\n") < 0) {
-    socket_setError(socket, "POSTFIX-POLICY: error sending data -- %s", STRERROR);
-    return FALSE;
-  }
+        if (Socket_print(socket,
+                         "request=smtpd_access_policy\n"
+                         "protocol_state=RCPT\n"
+                         "protocol_name=SMTP\n"
+                         "sender=monit@foo.tld\n"
+                         "recipient=monit@foo.tld\n"
+                         "client_address=1.2.3.4\n"
+                         "client_name=mx.foo.tld\n"
+                         "\n") < 0) {
+                THROW(IOException, "POSTFIX-POLICY: error sending data -- %s", STRERROR);
+        }
 
-  if(! socket_readln(socket, buf, sizeof(buf))) {
-    socket_setError(socket, "POSTFIX-POLICY: error receiving data -- %s", STRERROR);
-    return FALSE;
-  }
+        if (! Socket_readLine(socket, buf, sizeof(buf)))
+                THROW(IOException, "POSTFIX-POLICY: error receiving data -- %s", STRERROR);
 
-  Str_chomp(buf);
-
-  if( (strlen(buf) <= 7) || strncasecmp(buf, "action=", 7) ) {
-    socket_setError(socket, "POSTFIX-POLICY error: %s",
-      *buf?buf:"no action returned");
-    return FALSE;
-  }
-
-  return TRUE;
+        Str_chomp(buf);
 
+        if ((strlen(buf) <= 7) || strncasecmp(buf, "action=", 7))
+                THROW(IOException, "POSTFIX-POLICY error: %s", *buf ? buf : "no action returned");
 }
 
