@@ -66,7 +66,10 @@ static Protocol_T protocols[] = {
         &(struct Protocol_T){"GPS",             check_gps},
         &(struct Protocol_T){"RADIUS",          check_radius},
         &(struct Protocol_T){"MEMCACHE",        check_memcache},
-        &(struct Protocol_T){"WEBSOCKET",       check_websocket}
+        &(struct Protocol_T){"WEBSOCKET",       check_websocket},
+        &(struct Protocol_T){"REDIS",           check_redis},
+        &(struct Protocol_T){"MONGODB",         check_mongodb},
+        &(struct Protocol_T){"SIEVE",           check_sieve}
 };
 
 
