@@ -186,10 +186,9 @@ pid_t Util_getPid(char *pidfile);
 /**
  * Check whether the process is running
  * @param s The service being checked
- * @param refresh true to refresh the global ptree (useful for procmatch if process was mangled by monit in the same cycle such as by restart action) or false to use cached ptree
  * @return The PID of the running running process or 0 if the process is not running.
  */
-int Util_isProcessRunning(Service_T s, boolean_t refresh);
+int Util_isProcessRunning(Service_T s);
 
 
 /**
@@ -420,10 +419,10 @@ char *Util_portDescription(Port_T p, char *buf, int bufsize);
 
 
 /**
- *  Returns the FQDN hostname or fallback to gethostname() output
- *  @param buf the character array for hostname
- *  @param len the length of buf
- *  @return zero on success
+ * Returns the FQDN hostname or fallback to gethostname() output
+ * @param buf the character array for hostname
+ * @param len the length of buf
+ * @return zero on success
  */
 int Util_getfqdnhostname(char *buf, unsigned len);
 
@@ -436,5 +435,13 @@ int Util_getfqdnhostname(char *buf, unsigned len);
 const char *Util_timestr(int time);
 
 
+/**
+ * Parse the response from Monit HTTP interface.
+ * @param S Socket reference
+ * @exception IOException in the case of communication error or AssertException if monit returned application error.
+ */
+void Util_parseMonitHttpResponse(Socket_T S);
+
+
 #endif
 
